/*
 *  Ray++ - Object-oriented ray tracing library
 *  Copyright (C) 1998-2001 Martin Reinecke and others.
 *  See the AUTHORS file for more information.
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Library General Public
 *  License as published by the Free Software Foundation; either
 *  version 2 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Library General Public License for more details.
 *
 *  You should have received a copy of the GNU Library General Public
 *  License along with this library; if not, write to the Free
 *  Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  See the README file for more information.
 */

#include "lights/pointlight.h"

namespace RAYPP {

extern HANDLE<WORLD> World;

POINTLIGHT::POINTLIGHT ()
  : Location (0,0,0), Colour (1,1,1) {}

POINTLIGHT::POINTLIGHT (const VECTOR &Loc, const COLOUR &Col)
  : Location (Loc), Colour (Col) {}

//virtual
void POINTLIGHT::Init ()
  {
  if (initialized) return;
  World->Get_Surrounding_Volume (Location, MyVolume);
  initialized = true;
  }

//virtual
void POINTLIGHT::Transform (const TRANSFORM &trans)
  {
  cni();
  Location = trans.TransPoint (Location);
  }

//virtual
void POINTLIGHT::Cast_Light (const VECTOR &Pos, LIGHT_ARRAY &Arr) const
  {
  ci();
  COLOUR Col = Colour / (Pos - Location).SquaredLength();
  if (!Col.TooSmall()) Arr.push_back (LIGHT_ENTRY (Col, Location, MyVolume));
  }

void POINTLIGHT::Set_Colour (const COLOUR &new_col)
  {
  cni();
  Colour = new_col;
  }

void POINTLIGHT::Set_Location (const VECTOR &new_loc)
  {
  cni();
  Location = new_loc;
  }

} // namespace RAYPP
