/*
 *  Ray++ - Object-oriented ray tracing library
 *  Copyright (C) 1998-2001 Martin Reinecke and others.
 *  See the AUTHORS file for more information.
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Library General Public
 *  License as published by the Free Software Foundation; either
 *  version 2 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Library General Public License for more details.
 *
 *  You should have received a copy of the GNU Library General Public
 *  License along with this library; if not, write to the Free
 *  Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  See the README file for more information.
 */

#include "objects/simple_object.h"

namespace RAYPP {

SIMPLE_OBJECT::SIMPLE_OBJECT ()
  : Priority (-1) {}

//virtual
void SIMPLE_OBJECT::Init ()
  {
  if (initialized) return;

  if (Priority < 0) error ("object priority not set");

  if (Shape) Shape->Init();
    else error ("No shape specified");
  if (Surface) Surface->Init();
  if (Volume)  Volume->Init();

  if (!Shape->Has_Inside()) Volume.Clear();

  initialized = true;
  }

//virtual
void SIMPLE_OBJECT::Deinit ()
  {
  if (!initialized) return;

  Shape->Deinit();
  if (Surface) Surface->Deinit();
  if (Volume)  Volume ->Deinit();

  initialized = false;
  }

//virtual
void SIMPLE_OBJECT::Transform (const TRANSFORM &trans)
   {
   cni();
   if (  Shape) Shape  ->Transform (trans);
   if (Surface) Surface->Transform (trans);
   if ( Volume) Volume ->Transform (trans);
   }

//virtual
AXISBOX SIMPLE_OBJECT::BBox () const
  {
  ci();
  return Shape->BBox();
  }

//virtual
bool SIMPLE_OBJECT::Has_Volume () const
  {
  ci();
  return Volume;
  }

//virtual
bool SIMPLE_OBJECT::Volume_in_BBox () const
  {
  ci();
// ist das sicher richtig?
  if (Volume) return Shape->Inside_in_BBox();
  return true;
  }

//virtual
bool SIMPLE_OBJECT::Test (RAY &Ray, float8 &dist) const
  {
  ci();
  bool realhit (false);
  if (Ray.Inside.Priority > Priority) return false;
  bool tmp = Shape->Test (Ray, dist, realhit);
  if (tmp)
    if (realhit) Ray.maxdist = dist + 2*Small_dist;
  return tmp;
  }

//virtual
bool SIMPLE_OBJECT::Inside_Volume (const VECTOR &Loc,
  const INSIDE_INFO &nowInside, INSIDE_INFO &result) const
  {
  ci();
  if (!Volume) return false;
  if (nowInside.Priority > Priority) return false;
  if (!Shape->Inside (Loc)) return false;
  result.Priority = Priority;
  result.Vol = Volume;
  return true;
  }

//virtual
bool SIMPLE_OBJECT::Intersect (RAY &Ray, OBJECT_QUEUE &,
  INTERSECT_INFO &result) const
  {
  ci();
  if (Ray.Inside.Priority > Priority) return false;

  float8 depth;
  VECTOR Normal;

  if (!Shape->Intersect (Ray, depth, Normal)) return false;
  result.Has_Volume = Volume;
  result.Surf = Surface;
  result.depth = depth;
  result.Normal = Normal;
  result.Ins.Priority = Priority;
  result.Ins.Vol = Volume;
  return true;
  }

void SIMPLE_OBJECT::Set_Priority (uint4 &actcount) 
  {
  cni();
  Priority = ++actcount;
  }
void SIMPLE_OBJECT::Set_Shape (HANDLE<SHAPE> Shp)
  {
  cni();
  Shape = Shp;
  }
void SIMPLE_OBJECT::Set_Surface (HANDLE<SURFACE> Surf)
  {
  cni();
  Surface = Surf;
  }
void SIMPLE_OBJECT::Set_Volume (HANDLE<VOLUME> Vol)
  {
  cni();
  Volume = Vol;
  }

} // namespace RAYPP
