/*
 *  Ray++ - Object-oriented ray tracing library
 *  Copyright (C) 1998-2001 Martin Reinecke and others.
 *  See the AUTHORS file for more information.
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Library General Public
 *  License as published by the Free Software Foundation; either
 *  version 2 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Library General Public License for more details.
 *
 *  You should have received a copy of the GNU Library General Public
 *  License along with this library; if not, write to the Free
 *  Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  See the README file for more information.
 */

#include "kernel/globals.h"
#include "kernel/handle.h"
#include "worlds/scene.h"
#include "renderers/raytracer.h"
#include "outputs/tga_output.h"
#include "shapes/tube.h"
#include "shapes/cylinder.h"
#include "shapes/cone.h"
#include "shapes/plane.h"
#include "surfaces/matte.h"
#include "objects/simple_object.h"
#include "lights/pointlight.h"
#include "cameras/std_camera.h"


int main ()
  {
  using namespace RAYPP;

  HANDLE<OUTPUT> Output;

  Output = Cnew TGA_OUTPUT (640, 480, "conetest.tga");
  Output->Init();

  SCENE *Scn = Cnew SCENE;
  RAYTRACER *Raytracer = Cnew RAYTRACER;

  Raytracer->Add (Cnew STD_CAMERA (VECTOR (0,2.5,-6), VECTOR (0,0,0)));

  SIMPLE_OBJECT *obj;
  uint4 Priority = 1;

  obj = Cnew SIMPLE_OBJECT;
  obj->Set_Shape (Cnew CONE);
  obj->Translate(VECTOR(0, -0.5, 0));
  obj->Shear(0,0,1.5,0,0,0);
  obj->Scale(VECTOR(0.6, 1.0, 0.6));
  obj->Translate(VECTOR(1, 0, 0));
  obj->Set_Surface (Cnew MATTE (0.2, 1.0, COLOUR(1,0.5,0.5)));
  obj->Set_Priority (Priority);
  Scn->Add (obj);

  obj = Cnew SIMPLE_OBJECT;
  obj->Set_Shape (Cnew CONE);
  obj->Translate(VECTOR(0, -0.5, 0));
//  obj->Rotate(VECTOR(180, 0, 0));
  obj->Scale(VECTOR(0.6, 1.0, 0.6));
  obj->Translate(VECTOR(-1, 0, 0));
  obj->Set_Surface (Cnew MATTE (0.2, 1.0, COLOUR(0.5,1,0.5)));
  obj->Set_Priority (Priority);
  Scn->Add (obj);

  obj = Cnew SIMPLE_OBJECT;
  obj->Set_Shape (Cnew PLANE);
  obj->Translate(VECTOR(0, -2.0, 0));
  obj->Set_Surface (Cnew MATTE (0.2, 1.0, COLOUR(1,1,1)));
  obj->Set_Priority (Priority);
  Scn->Add (obj);

  Scn->Add (Cnew POINTLIGHT (VECTOR (0,7,-10), COLOUR (60,60,60)));
  Scn->Add (Cnew POINTLIGHT (VECTOR (-3,7,-2), COLOUR (40,40,40)));

  World = Scn;
  World->Init();
  Renderer = Raytracer;
  Renderer->Init();

  Output->Render();
  }
