/*
 *  Ray++ - Object-oriented ray tracing library
 *  Copyright (C) 1998-2001 Martin Reinecke and others.
 *  See the AUTHORS file for more information.
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Library General Public
 *  License as published by the Free Software Foundation; either
 *  version 2 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Library General Public License for more details.
 *
 *  You should have received a copy of the GNU Library General Public
 *  License along with this library; if not, write to the Free
 *  Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  See the README file for more information.
 */

#include "kernel/globals.h"
#include "kernel/handle.h"
#include "worlds/scene.h"
#include "renderers/raytracer.h"
#include "outputs/tga_output.h"
#include "shapes/sphere.h"
#include "shapes/box.h"
#include "shapes/csg_shape.h"
#include "surfaces/matte.h"
#include "objects/simple_object.h"
#include "lights/pointlight.h"
#include "lights/ambient_light.h"
#include "cameras/std_camera.h"


int main ()
  {
  using namespace RAYPP;

  HANDLE<OUTPUT> Output;
  Output = Cnew TGA_OUTPUT (640, 480, "csgtest.tga");
  Output->Init();

  SCENE *Scn = Cnew SCENE;
  RAYTRACER *Raytracer = Cnew RAYTRACER;

  Raytracer->Add (Cnew STD_CAMERA (VECTOR (0,0.75,-3.75), VECTOR (0,0,0)));

  SIMPLE_OBJECT *obj;
  uint4 Priority = 1;

  BOX *box = Cnew BOX (VECTOR(-0.5,-0.5,-1),VECTOR(0.5,0.5,0));
  box->Invert();

  CSG_SHAPE *csg = Cnew CSG_SHAPE (UNION);
  csg->Add (Cnew SPHERE (VECTOR ( 0.5,0,0), 1));
  csg->Add (Cnew SPHERE (VECTOR (-0.5,0,0), 1));
  CSG_SHAPE *csg2 = Cnew CSG_SHAPE (INTERSECTION);
  csg2->Add (csg);
  csg2->Add (box);

  obj = Cnew SIMPLE_OBJECT;
  obj->Set_Shape (csg2);
  obj->Set_Surface (Cnew MATTE (float4(0.3), float4(1.0), COLOUR(1,1,1)));
  obj->Set_Priority (Priority);
  Scn->Add (obj);

  Scn->Add (Cnew POINTLIGHT (VECTOR (10,30,-20), COLOUR (800,800,800)));
  Scn->Add (Cnew POINTLIGHT (VECTOR (-10,-30,-20), COLOUR (800,400,400)));
  Scn->Add (Cnew POINTLIGHT (VECTOR (0,0,-0.99), COLOUR (0.25,0.6,0.25)));

  World = Scn;
  World->Init();
  Renderer = Raytracer;
  Renderer->Init();

  Output->Render();
  }
