/*
 *  Ray++ - Object-oriented ray tracing library
 *  Copyright (C) 1998-2001 Martin Reinecke and others.
 *  See the AUTHORS file for more information.
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Library General Public
 *  License as published by the Free Software Foundation; either
 *  version 2 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Library General Public License for more details.
 *
 *  You should have received a copy of the GNU Library General Public
 *  License along with this library; if not, write to the Free
 *  Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  See the README file for more information.
 */

#include "kernel/globals.h"
#include "kernel/handle.h"
#include "worlds/scene.h"
#include "renderers/raytracer.h"
#include "outputs/tga_output.h"
#include "shapes/box.h"
#include "shapes/sphere.h"
#include "shapes/plane.h"
#include "surfaces/matte.h"
#include "surfaces/phong.h"
#include "surfaces/pigments/pure_colour.h"
#include "objects/simple_object.h"
#include "lights/pointlight.h"
#include "lights/ambient_light.h"
#include "cameras/std_camera.h"
#include "worlds/hmakers/pov_hmaker.h"


int main ()
  {
  using namespace RAYPP;

  HANDLE<OUTPUT> Output;
  Output = Cnew TGA_OUTPUT (640, 480, "phongtest.tga");
  Output->Init();

  RAYTRACER *Raytracer = Cnew RAYTRACER;

  SCENE *Scn = Cnew SCENE;

  Raytracer->Add (Cnew STD_CAMERA (VECTOR (0,0,-8), VECTOR (0,0,0)));

  SIMPLE_OBJECT *obj;
  uint4 Priority = 1;

  HANDLE<PIGMENT> Pig;
  Pig = Cnew PURE_COLOUR (COLOUR (1,1,1));

  for (int j=1;j<=3;j++)
    for (int i=1;i<=5;i++) 
    {
      obj = Cnew SIMPLE_OBJECT;
      obj->Set_Shape (Cnew SPHERE(VECTOR(-3.0+i,j-2,0), 0.45));
      obj->Set_Surface (Cnew PHONG (0.1, j*0.2, 1-j*0.2, 0,0, 5*i*i+5, Pig));
      obj->Set_Priority (Priority);
      Scn->Add (obj);
    }

  obj = Cnew SIMPLE_OBJECT;
  obj->Set_Shape (Cnew PLANE);
  obj->Set_Surface (Cnew MATTE (0, 1.0, Pig));
  obj->Set_Priority (Priority);
  obj->Translate (VECTOR (0,-2,0));
  Scn->Add (obj);

  Scn->Add (Cnew POINTLIGHT (VECTOR (-10,15,-20), COLOUR (600,300,300)));
  Scn->Add (Cnew POINTLIGHT (VECTOR (10,10,-20), COLOUR (300,300,600)));
  Scn->Add (Cnew AMBIENT_LIGHT (COLOUR (0.5,0.5,0.5)));

  Scn->Set_HMaker (Cnew POV_HMAKER);

  World = Scn;
  World->Init();

  Renderer = Raytracer;
  Renderer->Init();

  Output->Render();
  }
