/*
 *  Ray++ - Object-oriented ray tracing library
 *  Copyright (C) 1998-2001 Martin Reinecke and others.
 *  See the AUTHORS file for more information.
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Library General Public
 *  License as published by the Free Software Foundation; either
 *  version 2 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Library General Public License for more details.
 *
 *  You should have received a copy of the GNU Library General Public
 *  License along with this library; if not, write to the Free
 *  Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  See the README file for more information.
 */

/*! \file proj.cxx
    Test file for the PROJECTOR class
*/

/* 
    File written by Ugo Varetto - varetto@computer.org for testing
    the PROJECTOR class.
*/

#include "kernel/globals.h"
#include "kernel/handle.h"
#include "worlds/scene.h"
#include "renderers/raytracer.h"
#include "outputs/tga_output.h"
#include "shapes/plane.h"
#include "surfaces/matte.h"
#include "surfaces/phong.h"
#include "objects/simple_object.h"
#include "lights/pointlight.h"
#include "cameras/std_camera.h"
#include "shapes/sphere.h"
#include "surfaces/phong.h"
#include "surfaces/pigments/pure_colour.h"
#include "surfaces/pigments/std_pigments.h"
#include "volumes/clear_glass.h"

/////
#include "lights/projector.h"
/////

int main ()
  {
  using namespace RAYPP;

  HANDLE<OUTPUT> Output;

  Output = Cnew TGA_OUTPUT (640, 480, "proj.tga");
  Output->Init();

  RAYTRACER *Raytracer = Cnew RAYTRACER;
  SCENE *Scn = Cnew SCENE;

  Raytracer->Add (Cnew STD_CAMERA (VECTOR (0,6,-3), VECTOR (0,0,0)));

  SIMPLE_OBJECT *obj;
  uint4 Priority = 1;

  
  obj = Cnew SIMPLE_OBJECT;
  obj->Set_Shape (Cnew PLANE);
  obj->Translate(VECTOR(0,-2,0));
  obj->Set_Surface (Cnew MATTE (0.2, 1.0, COLOUR(1,1,1)));
  obj->Set_Priority (Priority);
  Scn->Add (obj);

  MATTE GlassP (0.2,1.0,COLOUR(1,1,1));
  HANDLE<SURFACE> Glass = Cnew MATTE(GlassP);
  HANDLE<VOLUME> GlassVol = Cnew CLEAR_GLASS(1.1);
  obj = Cnew SIMPLE_OBJECT;
  obj->Set_Shape (Cnew SPHERE(VECTOR(0,.5,0),1.2));
  obj->Translate(VECTOR(0,-2,0));
  obj->Set_Surface (Glass);
  obj->Set_Volume(GlassVol);
  obj->Set_Priority (Priority);
  Scn->Add (obj);

  Scn->Add (Cnew POINTLIGHT (VECTOR (0,6,-10), COLOUR (60,60,60)));

  /////////// 
  PROJECTOR *spot=Cnew PROJECTOR(
	  VECTOR (0,8,0.01),VECTOR(0,0,0),COLOUR (60,60,120),1,90);
  
  // The slide is 2 units far from light source
   float8 d=2; 

  // The image has its vertical axis aligned along Z
  spot->Set_ImageAxes(VECTOR(0,0,1),d); 

  // Choose the image to be displayed
  spot->Set_Image("logo.tga");

  // No transparent pixels
  spot->Set_Transparency(false);

  // Set the image size
  spot->Set_Size(1,1);  

  // Modulate the light colour with image pixel colour
  spot->Set_Mix(0); 
  
  // Insert projective spot into world
  Scn->Add (spot);
  //////////	
  
  World = Scn;
  World->Init();

  Renderer = Raytracer;
  Renderer->Init();

  Output->Render();
  }

