/*
 *  Ray++ - Object-oriented ray tracing library
 *  Copyright (C) 1998-2001 Martin Reinecke and others.
 *  See the AUTHORS file for more information.
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Library General Public
 *  License as published by the Free Software Foundation; either
 *  version 2 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Library General Public License for more details.
 *
 *  You should have received a copy of the GNU Library General Public
 *  License along with this library; if not, write to the Free
 *  Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  See the README file for more information.
 */

#include "surfaces/pigments/solid_pigment.h"

namespace RAYPP {

SOLID_PIGMENT::SOLID_PIGMENT ()
  : Turb (float4(0.0)), Lacunarity (float4(2.187)),
    Omega (float4(0.5)), Octaves (6) {}

//virtual
void SOLID_PIGMENT::Transform (const TRANSFORM &trans)
  {
  Trans.Add_Transform (trans);
  }

void SOLID_PIGMENT::Set_Turb (float4 newturb)
  { Turb = newturb; }

void SOLID_PIGMENT::Set_Lacunarity (float4 newlac)
  { Lacunarity = newlac; }

void SOLID_PIGMENT::Set_Omega (float4 newomega)
  { Omega = newomega; }

void SOLID_PIGMENT::Set_Octaves (uint1 newoct)
  { Octaves = newoct; }

void SOLID_PIGMENT::Set_CMap (const COLOURMAP &newmap)
  { CMap = newmap; }

} // namespace RAYPP
